
/*
   libgstroke - a GNOME stroke interface library
   Copyright (c) 1996,1997,1998,1999  Mark F. Willey, ETLA Technical

   See the files COPYRIGHT and LICENSE for distribution information.
*/

#include "gstroke.h"

#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <math.h>

/* determine which bin a point falls in */
guint stroke_bin (p_point point_p, guint bound_x_1, guint bound_x_2, guint bound_y_1, guint bound_y_2);

/* point list head and tail */
GSList pointList = NULL;

/* metrics for input stroke */
guint min_x = 10000;
guint min_y = 10000;
guint max_x = 0;
guint max_y = 0;
guint point_count = 0;

void stroke_init (void) {

  if (pointList != NULL) {
/* FIX: look up actual fn name and flags to recursively free */
    g_slist_free (pointArray)
  }
}

guint stroke_trans (GString sequence) {

  /* number of bins recorded in the stroke */
  guint sequence_count = 0;

  /* points-->sequence translation scratch variables */
  guint prev_bin = 0;
  guint current_bin = 0;
  guint bin_count = 0;

  /* flag indicating the start of a stroke - always count it in the sequence */
  guint first_bin = TRUE;

  /* bin boundary and size variables */
  guint delta_x, delta_y;
  guint bound_x_1, bound_x_2;
  guint bound_y_1, bound_y_2;


  /* determine size of grid */
  delta_x = max_x - min_x;
  delta_y = max_y - min_y;
  
  /* calculate bin boundary positions */
  bound_x_1 = min_x + (delta_x / 3);
  bound_x_2 = min_x + 2 * (delta_x / 3);

  bound_y_1 = min_y + (delta_y / 3);
  bound_y_2 = min_y + 2 * (delta_y / 3);

  if (delta_x > SCALE_RATIO * delta_y) {
      bound_y_1 = (max_y + min_y - delta_x) / 2 + (delta_x / 3);
      bound_y_2 = (max_y + min_y - delta_x) / 2 + 2 * (delta_x / 3);
  } else if (delta_y > SCALE_RATIO * delta_x) {
      bound_x_1 = (max_x + min_x - delta_y) / 2 + (delta_y / 3);
      bound_x_2 = (max_x + min_x - delta_y) / 2 + 2 * (delta_y / 3);
  }

/*
    printf ("DEBUG:: point count: %d\n",point_count);
    printf ("DEBUG:: min_x: %d\n",min_x);
    printf ("DEBUG:: max_x: %d\n",max_x);
    printf ("DEBUG:: min_y: %d\n",min_y);
    printf ("DEBUG:: max_y: %d\n",max_y);
    printf ("DEBUG:: delta_x: %d\n",delta_x);
    printf ("DEBUG:: delta_y: %d\n",delta_y);
    printf ("DEBUG:: bound_x_1: %d\n",bound_x_1);
    printf ("DEBUG:: bound_x_2: %d\n",bound_x_2);
    printf ("DEBUG:: bound_y_1: %d\n",bound_y_1);
    printf ("DEBUG:: bound_y_2: %d\n",bound_y_2);
*/

/*
   build string by placing points in bins, collapsing bins and discarding
   those with too few points...
*/

/* FIX - we want to iterate the point array, how to?? */
/*
  index = 0;
  while (p_point = g_ptr_array_next(index++,pointArray)) {
*/
  for (index = 0; index < pointArray->len; index++)
  point = g_ptr_array_index(index,pointArray);

    /* figure out which bin the point falls in */
    current_bin = stroke_bin(point_list_head,bound_x_1, bound_x_2, bound_y_1, bound_y_2);
    /* if this is the first point, consider it the previous bin, too. */
    prev_bin = (prev_bin == 0) ? current_bin : prev_bin;
/*    printf ("DEBUG:: current bin: %d\n",current_bin); */

    if (prev_bin == current_bin)
      bin_count++;
    else {  /* we are moving to a new bin -- consider adding to the sequence */
      if ((bin_count > (point_count * BIN_COUNT_PERCENT)) || (first_bin == TRUE)) {
        first_bin = FALSE;
        sequence[sequence_count++] = '0' + prev_bin;
/*        printf ("DEBUG:: adding sequence: %d\n",prev_bin);
*/
      }

      /* restart counting points in the new bin */
      bin_count=0;
      prev_bin = current_bin;
    }

    /* move to next point, freeing current point from list */
    point_list_tail = point_list_head;
    point_list_head = point_list_head->next;
    free (point_list_tail);
  }
  point_list_tail = NULL;

    /* add the last run of points to the sequence */
    sequence[sequence_count++] = '0' + current_bin;
/*    printf ("DEBUG:: adding final sequence: %d\n",current_bin); */

  /* bail out on error cases */
  if ((point_count < MIN_POINTS) || (sequence_count > MAX_SEQUENCE)) {
    point_count = 0;
    strcpy (sequence,"0");
    return FALSE;
  }

  /* add null termination and leave */
  point_count = 0;
  sequence[sequence_count] = '\0';
  return TRUE;
}

/* figure out which bin the point falls in */
guint stroke_bin (p_point point_p, guint bound_x_1, guint bound_x_2, guint bound_y_1, \
         guint bound_y_2) {

  guint bin_num = 1;
  if (point_p->x > bound_x_1) bin_num += 1;
  if (point_p->x > bound_x_2) bin_num += 1;
  if (point_p->y > bound_y_1) bin_num += 3;
  if (point_p->y > bound_y_2) bin_num += 3;

  return bin_num;
}

void stroke_record (guint x, guint y) {

  p_point new_point_p;
  guint delx, dely;
  float ix, iy;

  if (point_count < MAX_POINTS) {
    new_point_p = (p_point) malloc (sizeof(point));

    if (pointList == NULL) {

      /* first point in list - initialize metrics */
      min_x = 10000;
      min_y = 10000;
      max_x = 0;
      max_y = 0;
      point_count = 0;

    } else {

      /* interpolate between last and current point */
      delx = x - g_slist_last(pointList)->x;
      dely = y - g_slist_last(pointList)->y;

      if (abs(delx) > abs(dely)) {  /* step by the greatest delta direction */
        iy = g_slist_last(pointList)->y;

     /* go from the last point to the current, whatever direction it may be */
        for (ix = g_slist_last(pointList)->x; (delx > 0) ? (ix < x) : (ix > x); ix += (delx > 0) ? 1 : -1) {

          /* step the other axis by the correct increment */
          iy += fabs(((float) dely / (float) delx)) * (float) ((dely < 0) ? -1.0 : 1.0);

          /* add the interpolated point */
          new_point_p->x = ix;
          new_point_p->y = iy;
          g_slist_append(pointList,new_point_p);

          /* update metrics */
          if (((guint) ix) < min_x) min_x = (guint) ix;
          if (((guint) ix) > max_x) max_x = (guint) ix;
          if (((guint) iy) < min_y) min_y = (guint) iy;
          if (((guint) iy) > max_y) max_y = (guint) iy;
          point_count++;

          new_point_p = (p_point) malloc (sizeof(point));
        }
      } else {  /* same thing, but for dely larger than delx case... */
        ix = g_slist_last(pointList)->x;

        /* go from the last point to the current, whatever direction it may be
        */
        for (iy = g_slist_last(pointList)->y; (dely > 0) ? (iy < y) : (iy > y); iy += (dely > 0) ? 1 : -1) {

          /* step the other axis by the correct increment */
          ix += fabs(((float) delx / (float) dely)) * (float) ((delx < 0) ? -1.0 : 1.0);

          /* add the interpolated point */
          new_point_p->y = iy;
          new_point_p->x = ix;
          g_slist_append(pointList,new_point_p);

          /* update metrics */
          if (((guint) ix) < min_x) min_x = (guint) ix;
          if (((guint) ix) > max_x) max_x = (guint) ix;
          if (((guint) iy) < min_y) min_y = (guint) iy;
          if (((guint) iy) > max_y) max_y = (guint) iy;
          point_count++;

          new_point_p = (p_point) malloc (sizeof(point));
        }
      }

      /* add the sampled point */
      g_slist_append(pointList,new_point_p);
    }

    /* record the sampled point values */
    new_point_p->x = x;
    new_point_p->y = y;
  }
}

